#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""List field implementation code."""

from django import forms
from django.utils.translation import ugettext_lazy as _


class ListField(forms.Field):
    """List Field."""

    default_error_messages = {
        "invalid_list": _("Enter a list of values.")
    }

    def __init__(self, *args, **kwargs):
        """Init."""
        self.field = kwargs.pop("field", None) or forms.CharField()
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        """Normalize the value into python format with specified field."""
        if not value:
            return []
        elif not isinstance(value, (list, tuple)):
            raise forms.ValidationError(
                self.error_messages["invalid_list"], code="invalid_list"
            )
        errors = []
        normalize_values = []
        for (index, item) in enumerate(value):
            try:
                normalize_values.append(self.field.to_python(item))
            except forms.ValidationError as exc:
                errors.append(forms.ValidationError(
                    "Index %(index)d: %(err_msg)s", params={
                        "index": index,
                        "err_msg": ("").join(exc.messages),
                        "exception": exc
                    }
                ))
        if errors:
            raise forms.ValidationError(errors, code="invalid")
        return normalize_values

    # validate(self, value) function is reserved for the subclass of this
    # class.

    def run_validators(self, value):
        """Validate the value."""
        errors = []
        for (index, item) in enumerate(value):
            try:
                self.field.run_validators(item)
            except forms.ValidationError as exc:
                errors.append([
                    forms.ValidationError(
                        "Index %(index)d: %(err_msg)s", params={
                            "index": index,
                            "err_msg": ("").join(exc.messages),
                            "exception": exc
                        }
                    )
                ])
        if errors:
            raise forms.ValidationError(errors, code="invalid")



#!/usr/bin/env python
# coding=utf-8

"""Django AngularJS Helper Froms."""

import json
from datetime import date
from functools import wraps
from django import forms


class AngularForm(forms.Form):
    """AngularJS Form."""

    def __wrap_ng_init(self, f, fld):
        """Wrap ng-init attribute."""
        @wraps(f)
        def inside(name, value, attrs):
            format_func = getattr(
                fld, "ng_init_format_func",
                lambda v: (
                    fld.widget.format_value(v)
                    if isinstance(v, (date, str)) else
                    v
                )
            )
            model_var = \
                fld.widget.attrs.get("data-ng-model") or attrs["data-ng-model"]
            attrs["data-ng-init"] = \
                f"{model_var} = {json.dumps(format_func(value))}"
            return f(name, value, attrs)
        return inside

    def __init__(self, *args, **kwargs):
        """Init the function."""
        super().__init__(*args, **kwargs)
        metaclass = getattr(self, "Meta", type("Meta", (object,), {}))
        handle_ng_init = getattr(metaclass, "handle_ng_init", False)
        ng_init_format_func = getattr(metaclass, "ng_init_format_func", {})
        self.ng_model_prefix = getattr(metaclass, "ng_model_prefix", "model")

        for (name, field) in self.fields.items():
            model = f"{self.ng_model_prefix}.{name}"
            field.widget.attrs.setdefault("data-ng-model", model)
            if handle_ng_init:
                if name in ng_init_format_func:
                    setattr(
                        field, "ng_init_format_func",
                        ng_init_format_func[name]
                    )
                tmp = field.widget.get_context
                field.widget.get_context = self.__wrap_ng_init(tmp, field)



#!/usr/bin/env python
# coding=utf-8

"""Date Select for Angular Material."""

from django import forms
from .mdselect import MDSelect


class MDDateSelect(forms.SelectDateWidget):
    """Date Select for Angular Material."""

    select_widget = MDSelect



#!/usr/bin/env python
# coding=utf-8

"""MDMultiSelect module."""

from django.forms import SelectMultiple
from .mdselect import MDSelect


class MDMultiSelect(MDSelect, SelectMultiple):
    """MDSelect widget that allows multiple select."""

    allow_multiple_selected = True

    def get_context(self, name, value, attrs):
        """Override get_context."""
        context = super().get_context(name, value, attrs)
        context['widget']['attrs']['data-multiple'] = \
            bool(context['widget']['attrs'].pop("multiple"))
        return context



#!/usr/bin/env python
# coding=utf-8

"""Widgets for AngularJS (and Angular Material)."""

from .mdselect import MDSelect
from .mdmultiselect import MDMultiSelect
from .md_datepicker import MDDatePicker
from .md_dateselect import MDDateSelect
from .mdcheckbox import MDCheckBox

__all__ = (
    "MDSelect", "MDMultiSelect", "MDDatePicker", "MDDateSelect", "MDCheckBox"
)



#!/usr/bin/env python
# coding=utf-8

"""Base classes."""

import os

from django.forms.widgets import Widget
from django.forms.renderers import Jinja2 as Jinja2Base
from django.utils.safestring import mark_safe
from django.utils.functional import cached_property


class Jinja2Engine(Jinja2Base):
    """Jinja2 template engine."""

    @cached_property
    def engine(self):
        """Return engine setting."""
        return self.backend({
            'DIRS': [
                os.path.join(
                    os.path.dirname(os.path.abspath(__file__)),
                    self.backend.app_dirname
                )
            ],
            'APP_DIRS': True,
            'NAME': "djextra-widgets",
            'OPTIONS': {}
        })


class BaseWidget(Widget):
    """Base widget to render the widget properly."""

    def _render(self, template_name, context, renderer=None):
        """Override render protected function."""
        renderer = Jinja2Engine()
        template = renderer.get_template(template_name)
        return mark_safe(template.render(context))



#!/usr/bin/env python
# coding=utf-8

"""MDSelect widgets."""

from django.forms import Select

from .base import BaseWidget


class MDSelect(BaseWidget, Select):
    """MDSelect."""

    template_name = "md_select.html"
    option_template_name = "md_select_option.html"

    def __init__(self, disable_select=False, *args, **kwargs):
        """Init the class."""
        super().__init__(*args, **kwargs)
        self.disable_select = disable_select
        self.checked_attribute = {"data-selected": not self.disable_select}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Additional forms / form fields."""

from .listfield import ListField
from .forms import (AllRequiredForm, FieldAttributeForm)

__all__ = ("ListField", "AllRequiredForm", "FieldAttributeForm")



#!/usr/bin/env python
# coding=utf-8
"""Setup script."""

import sys
import os.path as path
from setuptools import setup, find_packages

dependencies = ["django>=1.11", "jinja2"]
name = "djextra"
desc = "Additional Functions for Django"
license = "MIT"
url = "https://github.com/hiroaki-yamamoto/djextra"
keywords = "Django"
version = "0.0.0"
version_file = path.join(path.abspath(path.dirname(__file__)), "VERSION")

if path.exists(version_file):
    with open(version_file) as v:
        version = v.read()

author = "Hiroaki Yamamoto"
author_email = "hiroaki@hysoftware.net"

if sys.version_info < (3, 6):
    raise RuntimeError("Not supported on earlier then python 3.6.")

try:
    with open('README.md') as readme:
        long_desc = readme.read()
except Exception:
    long_desc = None

setup(
    name=name,
    version=version,
    description=desc,
    long_description=long_desc,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=dependencies,
    zip_safe=False,
    author=author,
    author_email=author_email,
    license=license,
    keywords=keywords,
    url=url,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Framework :: Django :: 2.0",
    ]
)



#!/usr/bin/env python
# coding=utf-8

"""Django AngularJS Helper Froms."""

from functools import wraps
from django import forms


class AllRequiredForm(forms.Form):
    """All required form."""

    def __init__(self, *args, **kwargs):
        """Init field."""
        super().__init__(*args, **kwargs)
        optional_fields = getattr(
            getattr(self, "Meta", type("Meta", (object,), {})()),
            "optional", None
        ) or {}
        for (name, field) in self.fields.items():
            if name in optional_fields:
                continue
            field.required = True


class FieldAttributeForm(forms.Form):
    """
    Field common attribute form.

    This form enables to specify widget.attrs thru Meta class.

    Example:
    ```Python
    class TestForm(FieldAttributeForm):

        class Meta(object):
            common_attrs = {
                "data-on-load": "test()",
                "data-on-delay": lambda form, fld, name, value: value
            }
            fld_attrs = {
                "name1": {
                    "data-test": "test1"
                },
                "name2": {
                    "data-test": lambda form, fld, name, value: value
                }
            }

        name1 = forms.CharField(required=False)
        name2 = forms.CharField(required=False)
        number = forms.IntegerField(required=False)
    ```

    Above example shows how to handle attributes. Especially, data-on-delay
    and fld_attrs['name2'][data-test] attributes are evaluated when
    widget.get_context is called. Therefore, widget.get_context is wrapped
    to handle metadata attribute.
    """

    def __wrap_attr(self, f, fld):
        """Wrap attribute."""
        @wraps(f)
        def inside(name, value, attrs):
            metaclass = getattr(self, "Meta", type("Meta", (object,), {}))
            attrs.update(getattr(metaclass, "common_attrs", {}))
            attrs.update(getattr(metaclass, "fld_attrs", {}).get(name, {}))
            for (key, value_func) in attrs.items():
                try:
                    attrs[key] = value_func(self, fld, name, value)
                except TypeError:
                    continue
            return f(name, value, attrs)
        return inside

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        # print(self.fields)
        for name, fld in self.fields.items():
            tmp = fld.widget.get_context
            fld.widget.get_context = self.__wrap_attr(tmp, fld)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""AngularJS (aka Angular1, Angular Legacy) forms."""

from .forms import AngularForm
from .widgets import (
    MDSelect, MDMultiSelect, MDDatePicker, MDDateSelect, MDCheckBox
)


__all__ = (
    "AngularForm", "MDSelect", "MDMultiSelect",
    "MDDatePicker", "MDDateSelect", "MDCheckBox"
)



#!/usr/bin/env python
# coding=utf-8

"""Datepicker for Django."""

from django import forms

from .base import BaseWidget


class MDDatePicker(BaseWidget, forms.DateInput):
    """AngularJS Material DatePicker for Django."""

    template_name = "md_datepicker.html"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Extra functions for django."""

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjangoExtra(AppConfig):
    """Extra functions for Django."""

    name = "djextra"
    label = "djextra"
    verbose_name = _("Extra functions for Django")



#!/usr/bin/env python
# coding=utf-8

"""Selection Btn."""

from django import forms

from .base import BaseWidget


class MDCheckBox(BaseWidget, forms.CheckboxInput):
    """Material Checkbox."""

    template_name = "md_checkbox.html"

    def __init__(self, label="", help_text="", *args, **kwargs):
        """Init the instance."""
        super().__init__(*args, **kwargs)
        self.label = label
        self.help_text = help_text

    def get_context(self, name, value, attrs):
        """Override get_context."""
        ret = super().get_context(name, value, attrs)
        if "checked" in ret["widget"]["attrs"]:
            ret["widget"]["attrs"]["data-checked"] = \
                ret["widget"]["attrs"].pop("checked")
        ret["widget"]["label"] = self.label
        ret["widget"]["help_text"] = self.help_text
        return ret
